var UserAccountManager = {
    callbacks: {},
    
    login: function(_username, _password, successCallback, errorCallback) {
        Tunnel.log('login');
        var rnd = 'cb' + Math.random();
        UserAccountManager.callbacks[rnd] = {success:successCallback, error:errorCallback};
        Tunnel.out({event:'LOGIN',args:{username:_username, password:_password, id:rnd}});
    }, 
    
    logout: function(successCallback, errorCallback) {

    },
    
    callback: function(id) {

    }
};

window.Tunnel = window.Tunnel || {};

/* Functions that can be overridden by Android */
var javaScriptToNativeCode = function(jsonString, force) {
    if (!Tunnel.inUse || force) {
        Tunnel.inUse = true;
        window.location = Tunnel.PROTOCOL + encodeURIComponent(jsonString);
    } else {
        Tunnel.queue.push(jsonString);
    }
};
window.Tunnel.javaScriptToNativeCode = window.Tunnel.javaScriptToNativeCode || javaScriptToNativeCode;

/* Add common properties */
$.extend(window.Tunnel, {

    PROTOCOL : "tunnel://",
    inUse : true,
    queue : [],
    messageBuffer : [],
    logging : true,

    init : function() {
        Tunnel.inUse = false;
    },

    log : function(data) {
        if (Tunnel.logging) {
            var string = data;
            if (data instanceof Object) {
                string = JSON.stringify(data);
            }
            $('#log').prepend("<p>" + string + "</p>");
        }
    },

    // Callback function for WebView once it's processed message to then trigger the next.
    clear : function () {
        Tunnel.log('Tunnel cleared');
        Tunnel.inUse = false;
        setTimeout(Tunnel.processQueue, 10);
    },

    processQueue : function() {
        if (Tunnel.queue.length > 0) {
            Tunnel.javaScriptToNativeCode(Tunnel.queue.shift(), false);
        }
    },

    processBuffer : function() {
        if (Tunnel.messageBuffer.length > 0) {
            var message = Tunnel.messageBuffer.shift();
            //            Tunnel.log(message);
            //            Tunnel.log(message.event);
            // Fire Event through global backbone message bus
            Backbone.trigger(message.event, message.args);

            // Process next message in our input buffer
            Tunnel.processBuffer();
        }
    },

    inject: function(/*object*/ data) {
        if(navigator.userAgent.match(/(iPad|iPhone|iPod)/g)) {
            Tunnel.messageBuffer.push(data);
            //Tunnel.log(data);
            setTimeout(Tunnel.processBuffer, 10);
        }
        else if (navigator.userAgent.match(/android/gi)) {
            Backbone.trigger(data.event, data.args);
        }
    },

    out : function(data, force) {
        if (data instanceof Object) {
            Tunnel.javaScriptToNativeCode(JSON.stringify(data), force);
        }
        else {
            Tunnel.log("Can't send none Object data through the tunnel");
        }
    }
});

/* Default event bindings, to customize call Backbone.off('EVENT_NAME') and then rebind */

Backbone.on('DOWNLOAD_MAGAZINE', function(issueModel) {
    Tunnel.out({event:'DOWNLOAD_MAGAZINE', args:{'publication_issue_uuid':issueModel.get('publication_issue_uuid')}}
        );
});
Backbone.on('OPEN_SETTINGS', function() {
    Tunnel.out({event:'OPEN_SETTINGS', args:{}});
});

// Open native login screen
Backbone.on('OPEN_LOGIN', function() {
    Tunnel.out({event:'OPEN_LOGIN', args:{}});
});

// Ask for a magazine download to be resumed, magazine model would have been in BundleState.DOWNLOAD_PAUSED
Backbone.on('RESUME_DOWNLOAD_MAGAZINE', function(issueModel) {
    Tunnel.out({event:'RESUME_DOWNLOAD_MAGAZINE', args:{'publication_issue_uuid':issueModel.get('publication_issue_uuid')}}
        );
});

Backbone.on('OPEN_MAGAZINE', function(issueModel) {
    Tunnel.out({event:'OPEN_MAGAZINE', args:{'publication_issue_uuid':issueModel.get('publication_issue_uuid')}}
        );

});
Backbone.on('ARCHIVE_MAGAZINE', function(issueModel) {
    Tunnel.out({event:'ARCHIVE_MAGAZINE', args:{'publication_issue_uuid':issueModel.get('publication_issue_uuid')}}
        );
});

// Pass changes from the native model to the JS model which in turn my cause Views to be updated
Backbone.on('MAGAZINE_UPDATE DOWNLOAD_PROGRESS', function(args) {
    var updatedIssueModel = new Digifi.IssueModel(args);
    Digifi.library.add(updatedIssueModel, {merge: true });
});

// Trigger native subscription
Backbone.on('SUBSCRIBE', function(subscriptionModel) {
 Tunnel.out({event:'SUBSCRIBE', args:subscriptionModel});
});

// Pass changes from the native subscription model to the JS model
Backbone.on('SUBSCRIPTION_UPDATE', function(args) {
    var updatedSubscriptionModel = new Digifi.SubscriptionModel(args);
    Digifi.subscriptions.add(updatedSubscriptionModel, {merge: true }); 
});

Backbone.on('DIAGNOSTICS_UPDATE', function(args) {
    Digifi.diagnosticsModel.set(args);
});

// Pass changes from the native user model to the JS
Backbone.on('USER_UPDATE', function(args) {
    Tunnel.log('USER_UPDATE');
    Digifi.user.set(args);
});
Backbone.on('LOGIN_SUCCEEDED', function(args) {
    Tunnel.log('LOGIN_SUCCEEDED - callback_id: ' + args.callback_id);
    Digifi.user.set(args.user);
});
Backbone.on('LOGIN_FAILED', function(args) {
    Tunnel.log('LOGIN_FAILED - callback_id: ' + args.callback_id);
    if (args.user) {
        Digifi.user.set(args.user);
    }
});
Backbone.on('LOGIN_CALLBACK', function(args) {

});

Backbone.on('GET_PREVIEW_IMAGE', function(issueModel) {
   Tunnel.out({event:'GET_PREVIEW_IMAGE', args: {'publication_issue_uuid':issueModel.get('publication_issue_uuid')}});
});