// js/IssueView.js

var Digifi = Digifi || {};

Digifi.SubscriptionView = Backbone.View.extend({
    
    template: _.template( $( '#SubscriptionViewTemplate' ).html() ),
    
    events: {
        'click button.subscribe': 'subscribe'
    },
    
    initialize : function() {
        this.listenTo(this.model, 'change:price', this.render);
        this.listenTo(this.model, 'change:subscribed', this.render);
    },
    
    render: function() {
        this.$el.html( this.template( this.model.toJSON() ) );
        return this;
    },
    
    subscribe: function() {
        Backbone.trigger('SUBSCRIBE', this.model);
    }
    
});

Digifi.AboutView = Backbone.View.extend({
    template: _.template($('#AboutViewTemplate').html()),

    initialize: function() {
        this.listenTo(this.model, 'change:appVersion', this.appVersionChanged);
    },

    appVersionChanged: function() {
        this.$el.find('.appVersion').text(this.model.get('appVersion'));
    },

    render: function() {
        this.$el.html( this.template( this.model.toJSON() ) );
        return this;
    }

});

Digifi.DiagnosticsView = Backbone.View.extend({
    template: _.template($('#DiagnosticsViewTemplate').html()),

    events: {
        'submit form': 'sendDiagnostics',
        'change .check' : 'loggingSwitchChanged'
    },

    initialize: function() {
        this.listenTo(this.model, 'change:submittingLogs', this.submittingLogsChanged);
    },

    submittingLogsChanged: function() {
        var submittingLogs = this.model.get('submittingLogs');
        if(submittingLogs) {
            this.$el.find('.sendLogsButton > img').css('visibility', 'visible');
        } else {
            this.$el.find('.sendLogsButton > img').css('visibility', 'hidden');
        }
    },

    loggingSwitchChanged: function() {
        var isChecked = this.$el.find('.check').prop('checked');
        if(isChecked) {
            this.model.set('enabled', 1);
        } else {
            this.model.set('enabled', 0);
        }
    },

    sendDiagnostics: function(e) {
        e.preventDefault();
        var feedbackText = $(".diagnosticsFeedback textarea").val();
        var email = $(".emailInputField").val();

        //Input validation
        if(feedbackText === "" || email === "") {
            alert("Please fill in your e-mail and tell us what went wrong in order to send the application logs");
            return;
        }
        if(!$(".emailInputField")[0].checkValidity()) {
            alert("The email address is not valid");
            return;
        }

        this.model.set('submittingLogs', true);

        Tunnel.out({event:'SEND_DIAGNOSTICS', args: {
            "notes" : feedbackText,
            "email" : email 
        }});
    },

    render: function() {
        this.$el.html( this.template( this.model.toJSON() ) );
        return this;
    }
});

Digifi.MobileFullscreenView = Backbone.View.extend({

    template: _.template( $('#MobileViewTemplate').html()),

    events: {
        'click .backButton' : 'backButton'
    },

    backButton: function(e) {
        e.preventDefault();
        this.$el.hide();
    },

    showAbout: function() {
        this.$el.find(".mobileContent").hide();
        this.$el.find(".aboutContent").show();
        this.$el.show();
    },

    showDiagnostics: function() {
        this.$el.find(".mobileContent").hide();
        this.$el.find(".diagnosticsContent").show();
        this.$el.show();
    },

    showHowto: function() {
        this.$el.find(".mobileContent").hide();
        this.$el.find(".howtoContent").show();
        var howToImage = this.$el.find(".howtoContent > img");
        if(howToImage.length === 0) {
            var img = $("<img>");
            img.attr("src", "img/howto-portrait.jpg");
            this.$el.find(".howtoContent").append(img);
        }
        this.$el.show();
    },

    showFaq: function() {
        window.open('http://digifi.it/faqs/');
    },

    render: function() {
        this.$el.html( this.template( this.model.toJSON() ) );

        // Render sub-views
        var aboutView = new Digifi.AboutView({ el: this.$el.find(".aboutContent"), model: this.model });
        aboutView.render();

        var diagnosticsView = new Digifi.DiagnosticsView({ el: this.$el.find(".diagnosticsContent"), model: this.model });
        diagnosticsView.render();        

        return this;
    }
});

Digifi.MobileSettingsView = Backbone.View.extend({

    template: _.template( $('#SideMenuViewTemplate').html()),

    events : {
        'click .backButton' : 'backButton',
        'click .about': 'about',
        'click .howto': 'howto',
        'click .faq': 'faq',
        'click .diagnostics': 'diagnostics'
    },

    backButton: function(e) {
        e.preventDefault();
        this.$el.hide();
    },

    about: function(e) {
        e.preventDefault();
        Digifi.mobileFullscreenView.showAbout();
    },

    howto: function(e) {
        e.preventDefault();
        Digifi.mobileFullscreenView.showHowto();
    },

    faq: function(e) {
        e.preventDefault();
        Digifi.mobileFullscreenView.showFaq();
    },

    diagnostics: function(e) {
        e.preventDefault();
        Digifi.mobileFullscreenView.showDiagnostics();
    },

    render: function() {
        var templateJSON = this.model.toJSON();
        templateJSON.hasBackButton = true;
        this.$el.html(this.template(templateJSON));
        return this;
    }
});

Digifi.SettingsView = Backbone.View.extend({
    
    template: _.template( $( '#SettingsViewTemplate' ).html() ),

    events : {
        'click .about': 'about',
        'click .howto': 'howto',
        'click .faq': 'faq',
        'click .diagnostics': 'diagnostics'
    },

    about: function () {
        this.$el.find('.settingsTitleText').text('About');
        $(".settingsSideMenu span").removeClass("selectedSettings");
        this.$el.find('.about').addClass("selectedSettings");
        this.$el.find('.settingsContent').hide();
        this.$el.find('.aboutContent').show();
    },

    howto: function() {
        this.$el.find('.settingsTitleText').text('How to use');
        $(".settingsSideMenu span").removeClass("selectedSettings");
        this.$el.find('.howto').addClass("selectedSettings");
        this.$el.find('.settingsContent').hide();
        this.$el.find('.howtoContent').show();
    },

    faq: function() {
        this.$el.find('.settingsTitleText').text('Frequently Asked Questions');
        $(".settingsSideMenu span").removeClass("selectedSettings");
        this.$el.find('.faq').addClass("selectedSettings");
        this.$el.find('.settingsContent').hide();
        this.$el.find('.faqContent').show();
        var iframe = this.$el.find('.faqContent iframe');
        if(typeof iframe.attr("src") === "undefined") {
            iframe.attr('src', 'http://digifi.it/faqs/');
        };
    },

    diagnostics: function() {
        this.$el.find('.settingsTitleText').text('Diagnostics');
        $(".settingsSideMenu span").removeClass("selectedSettings");
        this.$el.find('.diagnostics').addClass("selectedSettings");
        this.$el.find('.settingsContent').hide();
        this.$el.find('.diagnosticsContent').show();
    },
    
    render: function() {
        // Render main-template
        var templateJSon = this.model.toJSON();
        templateJSon.sideMenuViewTemplate = _.template( $('#SideMenuViewTemplate').html(), { hasBackButton: false} );
        this.$el.html(this.template(templateJSon));

        // Render sub-views
        var aboutView = new Digifi.AboutView({ el: this.$el.find(".aboutContent"), model: this.model });
        aboutView.render();
        var diagnosticsView = new Digifi.DiagnosticsView({ el: this.$el.find(".diagnosticsContent"), model: this.model });
        diagnosticsView.render();

        // Render about page by default
        this.about();
        return this;
    }    
});

Digifi.IssueView = Backbone.View.extend({
    tagName : 'div',
    attributes : function () {
        return {
            class : this.model.get('is_current_issue') ? 'currentIssue issueContainer' : 'issueContainer',
            id : this.model.get('publication_issue_uuid')
        };
    },

    template: _.template( $( '#IssueViewTemplate' ).html() ),
    
    events : {
        'click .archive': 'archive',
        'click .download': 'download',
        'click .view': 'view',
        'click .resume': 'resume',
        'click .infoIcon': 'displayInfo'
    },
    
    initialize : function() {
        this.listenTo(this.model, 'REMOVED', this.dropped);
        this.listenTo(this.model, 'change:title', this.titleChanged);
        this.listenTo(this.model, 'change:pretty_progress', this.prettyProgressChanged);
        this.listenTo(this.model, 'change:progress', this.progressChanged);
        this.listenTo(this.model, 'change:bundle_state', this.bundleStateChanged);
        this.listenTo(this.model, 'change:preview_src', this.previewChanged);
        this.listenTo(this.model, 'change:coverflow_src', this.coverflowChanged);
        this.listenTo(this.model, 'change:user_can_download', this.userCanDownloadChanged);
        this.listenTo(this.model, 'change:is_free', this.isFreeChanged);
    },

    currentProgressDownload : 0,

    render: function() {
        // Make sure that row height is resized correctly when the preview image is loaded
        var previewImg = new Image();
        var self = this;
        $(previewImg).load(function() {
            // Explicitly set the height of the container to fit its child, with preview image
            // After that, make sure it is consistent with the current row
            self.$el.height(self.$el.children().height());
            equalizeIssueContainersHeights();
        });
        previewImg.src = this.model.get('preview_src');

        this.$el.html(this.template(this.model.toJSON()));
        this.$el.find(".coverImage").append(previewImg);

        this.bundleStateChanged(); 
        return this;
    },
    
    bundleStateChanged: function() {
        Tunnel.log(this.model.get('publication_issue_uuid') + ' download state= ' + this.model.get('bundle_state'));
        switch (this.model.get('bundle_state')) {
            case BundleState.DOWNLOAD_PAUSED:
                this.$('.bundleState').html('DOWNLOAD_PAUSED');
                break;
            case BundleState.DOWNLOADING:
                this.$('.bundleState').html('DOWNLOADING');
                break;
            case BundleState.NOT_DOWNLOADED:
                this.$('.bundleState').html('NOT_DOWNLOADED');
                break;
            case BundleState.DOWNLOADED:
                this.$('.bundleState').html('DOWNLOADED');
                break;
            case BundleState.UNZIPPING:
                this.$('.bundleState').html('UNZIPPING');
            default:
                this.$('.bundleState').html('UNKNOWN');
                break;
        }
        if (this.model.get('bundle_state') == BundleState.DOWNLOADED) {
            // XXX $ = jQuery this.$ jQuery in local context not of the document
            this.$('.actionButton, .infoButton').hide();
            this.$('.progressIndicator').hide();
            this.$('.viewarchive').show();
            this.$('.view').show();
            this.$('.archive').show();
        }
        if (this.model.get('bundle_state') == BundleState.NOT_DOWNLOADED) {
            this.$('.actionButton, .infoButton').hide();
            this.$('.download').show();
        }
        if (this.model.get('bundle_state') == BundleState.DOWNLOAD_PAUSED) {
            this.$('.actionButton, .infoButton').hide();
            this.$('.resume').show();
        }
        if (this.model.get('bundle_state') == BundleState.DOWNLOADING) {
            this.$('.actionButton, .infoButton').hide();
            this.$('.downloading').show();
            this.$('.progressIndicator').show();
            this.progressChanged();
        }
        if(this.model.get('bundle_state') == BundleState.UNZIPPING) {
            this.$('.progressIndicator > svg').hide();
            this.$('.actionButton, .infoButton').hide();
            this.$('.unzipping').show();
            this.$('.preparing').show();
        }
    },
    
    dropped: function() {
        // Destroy the view for the removed model
        this.remove();   
    },
    
    resume: function() {
        console.log('download ' + this.model.get('title'));
        Backbone.trigger('RESUME_DOWNLOAD_MAGAZINE', this.model);
    },
    
    download: function() {
        console.log('download ' + this.model.get('title'));
        Backbone.trigger('DOWNLOAD_MAGAZINE', this.model);
    },
    
    view: function() {
        Backbone.trigger('OPEN_MAGAZINE', this.model);
    },
        
    archive: function() {
        console.log('archive ' + this.model.get('title'));
        Backbone.trigger('ARCHIVE_MAGAZINE', this.model);
    },

    displayInfo: function() {
        this.$('.infoContainer').toggle();
        this.$('.progressIndicator').toggle();
    },
    
    titleChanged: function() {
        console.log('title changed');
    },
    
    userCanDownloadChanged: function() {
        this.$('.userCanDownload').text('user can download = ' + this.model.get('user_can_download'));
    },

    updatePercentageState: function(percentage) {
        this.currentProgressDownload++;
    
        var alpha = (this.currentProgressDownload * 360) / 100;
        var r = ( alpha * Math.PI / 180 )
            , x = Math.sin( r ) * 70
            , y = Math.cos( r ) * - 70
            , mid = ( alpha > 180 ) ? 1 : 0
            , anim = 'M 0 0 v -70 A 70 70 1 ' 
                + mid + ' 1 ' 
                +  x  + ' ' 
                +  y  + ' z';
 
        this.$('.loader').attr( 'd', anim); 
  
        if(this.currentProgressDownload < percentage) {
            var self = this;
            setTimeout(function() {
                self.updatePercentageState(percentage);
            }, 30);
        }
    },

    prettyProgressChanged: function() {
        this.$('.downloading').text(this.model.get('pretty_progress'));
    },
    
    progressChanged: function() {
        var p = Number(this.model.get('progress'));
        var x = (p*100);
        this.updatePercentageState(x);
    },

    coverflowChanged: function() {
        this.$('#coverflow').attr('src', this.model.get('coverflow_src'));
    },
    
    previewChanged: function() {
        //Tunnel.log("preview changed" + this.model.get('preview_src'));
        if (this.model.get('preview_src').length > 100) {
            this.$('.coverImage img').attr('src', this.model.get('preview_src'));
        }
    },
    
    isFreeChanged: function() {
        this.$('.isFree').text('Is free = ' + this.model.get('is_free'));
    }
});