module.exports = function(grunt) {
  var pkgJson = grunt.file.readJSON('package.json');
  var packageVersion = grunt.option("setVersion") || pkgJson.version;

  // Project configuration.
  grunt.initConfig({
    version: packageVersion,
    name: pkgJson.name,
    clean: ["dist/css/*.css, dist/css/*.map"],
    shell: {
        bower: {
            command: 'bower install'
        }
    },
    bower_postinst: {
      dist: {
        options: {
          components: {
            // Build our branded library core, passing the customized colors using our less file
            'branded-library-core': ['npm', {'grunt': ['default', '--customColors=../../css/branded-colors.less']}]
          }
        }
      }
    },
    bowercopy: {
      options: {
        srcPrefix: 'bower_components'
      },
      scripts: {
          options: {
            destPrefix: 'js/libs'
          },
          files: {
            'backbone-min.js': 'backbone/backbone-min.js',
            'jquery.min.js': 'jquery/jquery.min.js',
            'underscore-min.js': 'underscore/underscore-min.js'
          }
      },
      css: {
        options: {
          destPrefix: 'css/libs'
        },
        files: {
            'normalize.css': 'normalize.css/normalize.css',

        }
      },
      core: {
        files : {
          'core': 'branded-library-core/dist'
        }
      }
    },
    less: {
       production: {
        files: {
          "dist/css/branded.css": ["css/branded.less"]
        },
        options: {
          compress: true,
          sourceMap: true,
          sourceMapFilename: 'dist/css/branded.css.map',
          sourceMapRootpath: '../../'
        }
      }
    },
    watch: {
      minified: {
        files: ['index.html','dist/css/branded.css', 'core/css/main.css'],
        options: {
          livereload: 35729
        }
      },
      branded: {
        files: ['css/branded.less'],
        tasks: ['less']
      },
      brandedColors: {
        files: ['css/branded-colors.less'],
        tasks: ['default']
      }
    },
    express: {
      all: {
        options: {
          port: 9000,
          hostname: "0.0.0.0",
          bases: [__dirname],
          livereload: 35729
        }
      }
    },
    open: {
      all: {
        // Gets the port from the connect configuration
        path: 'http://localhost:<%= express.all.options.port%>'
      }
    },
	bumpup: ['package.json', 'bower.json'],
    exec: {
      svn_commit: 'svn commit -m "NA: Going to release version <%= version %>"'
    },
    svn_tag: {
      options: {
        tag: '<%= version %>',
        commitMessage: 'NA: creating tag for release <%= version %>',
        projectRoot: 'svn+ssh://svn@svn.aptusinteractive.com/svn/mobileapps/branded-library-customizations/<%= name %>'
      }
    },
    parallel: {
      watch: {
        options: {
          stream: true
        },
        tasks: [{
          grunt: true,
          args: ['watch:minified']
        }, {
          grunt: true,
          args: ['watch:branded']
        }, {
          grunt: true,
          args: ['watch:brandedColors']
        }]
      },
    }
});

  grunt.loadNpmTasks('grunt-shell');
  grunt.loadNpmTasks('grunt-bowercopy');
  grunt.loadNpmTasks('grunt-contrib-clean');
  grunt.loadNpmTasks('grunt-bower-postinst');
  grunt.loadNpmTasks('grunt-contrib-less');
  grunt.loadNpmTasks('grunt-express');
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-open');
  grunt.loadNpmTasks('grunt-bumpup');
  grunt.loadNpmTasks('grunt-exec');
  grunt.loadNpmTasks('grunt-svn-tag');
  grunt.loadNpmTasks('grunt-parallel');

  // Default task(s).
  grunt.registerTask('default', ['clean', 'bower_postinst', 'bowercopy', 'less']);

  // Creates the `server` task
  grunt.registerTask('server', ['default', 'express', 'open', 'parallel']);

  // Creates the 'release' task
  grunt.registerTask('release', ['default', 'bumpup:' + packageVersion, 'exec', 'svn_tag']);

};
